@@ -19,7 +19,6 @@
 #include "zarray_impl.hpp"
 #include "zassign.hpp"
 #include "zfunction.hpp"
-#include "zview.hpp"
 #include "zmath.hpp"
 
 namespace xt
@@ -81,8 +80,8 @@ namespace xt
         template <class T>
         const xarray<T>& get_array() const;
 
-        const dynamic_shape<std::ptrdiff_t>& get_strides() const;
-        layout_type layout() const;
+        void assign_to(zarray_impl& dst) const;
+
         std::size_t dimension() const;
         const shape_type& shape() const;
         void resize(const shape_type& shape);
@@ -102,6 +101,12 @@ namespace xt
         template <class E>
         void init_implementation(const xexpression<E>& e, zarray_expression_tag);
 
+        template <class E>
+        zarray& assign_expression(const xexpression<E>& e, xtensor_expression_tag);
+
+        template <class E>
+        zarray& assign_expression(const xexpression<E>& e, zarray_expression_tag);
+
         implementation_ptr p_impl;
     };
 
@@ -122,6 +127,19 @@ namespace xt
         semantic_base::assign(e);
     }
 
+    template <class E>
+    inline zarray& zarray::assign_expression(const xexpression<E>& e, xtensor_expression_tag)
+    {
+        zarray tmp(e);
+        return (*this = tmp);
+    }
+
+    template <class E>
+    inline zarray& zarray::assign_expression(const xexpression<E>& e, zarray_expression_tag)
+    {
+        return semantic_base::operator=(e);
+    }
+
     inline zarray::zarray(implementation_ptr&& impl)
         : p_impl(std::move(impl))
     {
@@ -177,7 +195,7 @@ namespace xt
     template <class E>
     inline zarray& zarray::operator=(const xexpression<E>& e)
     {
-        return semantic_base::operator=(e);
+        return assign_expression(e, extension::get_expression_tag_t<std::decay_t<E>>());
     }
 
     inline void zarray::swap(zarray& rhs)
@@ -207,21 +225,17 @@ namespace xt
         return dynamic_cast<const ztyped_array<T>*>(p_impl.get())->get_array();
     }
 
+    inline void zarray::assign_to(zarray_impl& dst) const
+    {
+        dst.resize(shape());
+        zdispatcher_t<detail::xassign_dummy_functor, 1>::dispatch(get_implementation(), dst);
+    }
+
     inline std::size_t zarray::dimension() const
     {
         return p_impl->dimension();
     }
 
-    inline auto zarray::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
-    {
-        return p_impl->get_strides();
-    }
-
-    inline layout_type zarray::layout() const
-    {
-        return p_impl->layout();
-    }
-
     inline auto zarray::shape() const -> const shape_type&
     {
         return p_impl->shape();
@@ -257,9 +271,10 @@ namespace xt
         return p_impl->set_metadata(metadata);
     }
 
-    inline zview<const zarray&> make_strided_view(const zarray& z, xstrided_slice_vector& slices)
+    inline zarray strided_view(zarray& z, xstrided_slice_vector& slices)
     {
-        return zview<const zarray&>(z, slices);
+        std::unique_ptr<zarray_impl> p(z.get_implementation().strided_view(slices));
+        return zarray(std::move(p));
     }
 }
 
