@@ -17,9 +17,97 @@
 #include <xtensor/xnoalias.hpp>
 #include <xtensor/xscalar.hpp>
 #include <xtensor/xshape.hpp>
+#include <xtensor/xshape.hpp>
+#include "xtensor/xstrided_view.hpp"
 
 namespace xt
 {
+    template <class CTE>
+    class zarray_wrapper;
+
+    template <class CTE>
+    class zchunked_wrapper;
+
+    template <class CTE>
+    class zexpression_wrapper;
+
+    namespace detail
+    {
+        template <class E>
+        struct is_xstrided_view : std::false_type
+        {
+        };
+
+        template <class CT, class S, layout_type L, class FST>
+        struct is_xstrided_view<xstrided_view<CT, S, L, FST>> : std::true_type
+        {
+        };
+
+        template <class CT>
+        using is_const = std::is_const<std::remove_reference_t<CT>>;
+
+        template <class CT, class R = void>
+        using disable_xstrided_view_t = std::enable_if_t<!is_xstrided_view<CT>::value || is_const<CT>::value, R>;
+
+        template <class CT, class R = void>
+        using enable_const_t = std::enable_if_t<is_const<CT>::value, R>;
+
+        template <class CT, class R = void>
+        using disable_const_t = std::enable_if_t<!is_const<CT>::value, R>;
+    }
+
+    /******************
+     * zarray builder *
+     ******************/
+
+    namespace detail
+    {
+        template <class E>
+        struct is_xarray : std::false_type
+        {
+        };
+
+        template <class T, layout_type L, class A, class SA>
+        struct is_xarray<xarray<T, L, A, SA>> : std::true_type
+        {
+        };
+
+        template <class E>
+        struct is_chunked_array : std::false_type
+        {
+        };
+
+        template <class CS>
+        struct is_chunked_array<xchunked_array<CS>> : std::true_type
+        {
+        };
+
+        template <class E>
+        struct zwrapper_builder
+        {
+            using closure_type = xtl::closure_type_t<E>;
+            using wrapper_type = std::conditional_t<is_xarray<std::decay_t<E>>::value,
+                                                    zarray_wrapper<closure_type>,
+                                                    std::conditional_t<is_chunked_array<std::decay_t<E>>::value,
+                                                                       zchunked_wrapper<closure_type>,
+                                                                       zexpression_wrapper<closure_type>
+                                                                      >
+                                                    >;
+
+            template <class OE>
+            static wrapper_type* run(OE&& e)
+            {
+                return new wrapper_type(std::forward<OE>(e));
+            }
+        };
+
+        template <class E>
+        inline auto build_zarray(E&& e)
+        {
+            return zwrapper_builder<E>::run(std::forward<E>(e));
+        }
+    }
+
     const std::string endianness_string = (xtl::endianness() == xtl::endian::little_endian) ? "<" : ">";
 
     template <class T>
@@ -145,9 +233,7 @@ namespace xt
 
         virtual self_type* clone() const = 0;
 
-        virtual const dynamic_shape<std::ptrdiff_t>& get_strides() const = 0;
-        virtual std::size_t get_offset() const = 0;
-        virtual layout_type layout() const = 0;
+        virtual self_type* strided_view(xstrided_slice_vector& slices) = 0;
 
         virtual const nlohmann::json& get_metadata() const = 0;
         virtual void set_metadata(const nlohmann::json& metadata) = 0;
@@ -179,9 +265,13 @@ namespace xt
 
         virtual ~ztyped_array() = default;
 
+        virtual bool is_array() const = 0;
+
         virtual xarray<T>& get_array() = 0;
         virtual const xarray<T>& get_array() const = 0;
 
+        virtual void assign(xarray<T>&& rhs) = 0;
+
         XTL_IMPLEMENT_INDEXABLE_CLASS()
 
     protected:
@@ -209,14 +299,16 @@ namespace xt
 
         virtual ~zexpression_wrapper() = default;
 
+        bool is_array() const override;
+
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
 
+        void assign(xarray<value_type>&& rhs) override;
+
         self_type* clone() const override;
 
-        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
-        std::size_t get_offset() const override;
-        layout_type layout() const override;
+        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -232,6 +324,27 @@ namespace xt
 
         void compute_cache() const;
 
+        zarray_impl* strided_view_impl(xstrided_slice_vector& slices, std::true_type);
+        zarray_impl* strided_view_impl(xstrided_slice_vector& slices, std::false_type);
+
+        template <class CT>
+        using enable_assignable_t = enable_assignable_expression<CT, xarray<value_type>>;
+
+        template <class CT>
+        using enable_not_assignable_t = enable_not_assignable_expression<CT, xarray<value_type>>;
+
+        template <class CT = CTE>
+        enable_assignable_t<CT> assign_impl(xarray<value_type>&& rhs);
+        
+        template <class CT = CTE>   
+        enable_not_assignable_t<CT> assign_impl(xarray<value_type>&& rhs);
+
+        template <class CT = CTE>
+        enable_assignable_t<CT> resize_impl();
+
+        template <class CT = CTE>
+        enable_not_assignable_t<CT> resize_impl();
+
         CTE m_expression;
         mutable xarray<value_type> m_cache;
         mutable bool m_cache_initialized;
@@ -259,14 +372,16 @@ namespace xt
 
         virtual ~zscalar_wrapper() = default;
 
+        bool is_array() const override;
+
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
 
+        void assign(xarray<value_type>&& rhs) override;
+
         self_type* clone() const override;
 
-        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
-        std::size_t get_offset() const override;
-        layout_type layout() const override;
+        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -279,7 +394,7 @@ namespace xt
     private:
 
         zscalar_wrapper(const zscalar_wrapper&) = default;
-
+        
         CTE m_expression;
         xarray<value_type> m_array;
         nlohmann::json m_metadata;
@@ -304,14 +419,16 @@ namespace xt
 
         virtual ~zarray_wrapper() = default;
 
+        bool is_array() const override;
+
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
 
+        void assign(xarray<value_type>&& rhs) override;
+
         self_type* clone() const override;
 
-        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
-        std::size_t get_offset() const override;
-        layout_type layout() const override;
+        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -325,6 +442,12 @@ namespace xt
 
         zarray_wrapper(const zarray_wrapper&) = default;
 
+        template <class CT = CTE>
+        detail::enable_const_t<CT> assign_impl(xarray<value_type>&& rhs);
+
+        template <class CT = CTE>
+        detail::disable_const_t<CT> assign_impl(xarray<value_type>&& rhs);
+
         CTE m_array;
         nlohmann::json m_metadata;
     };
@@ -359,14 +482,16 @@ namespace xt
 
         virtual ~zchunked_wrapper() = default;
 
+        bool is_array() const override;
+
         xarray<value_type>& get_array() override;
         const xarray<value_type>& get_array() const override;
 
+        void assign(xarray<value_type>&& rhs) override;
+
         self_type* clone() const override;
 
-        const dynamic_shape<std::ptrdiff_t>& get_strides() const override;
-        std::size_t get_offset() const override;
-        layout_type layout() const override;
+        zarray_impl* strided_view(xstrided_slice_vector& slices) override;
 
         const nlohmann::json& get_metadata() const override;
         void set_metadata(const nlohmann::json& metadata) override;
@@ -384,6 +509,12 @@ namespace xt
 
         void compute_cache() const;
 
+        template <class CT = CTE>
+        detail::enable_const_t<CT> assign_impl(xarray<value_type>&& rhs);
+
+        template <class CT = CTE>
+        detail::disable_const_t<CT> assign_impl(xarray<value_type>&& rhs);
+        
         CTE m_chunked_array;
         shape_type m_chunk_shape;
         mutable xarray<value_type> m_cache;
@@ -411,85 +542,96 @@ namespace xt
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::get_array() -> xarray<value_type>&
+    bool zexpression_wrapper<CTE>::is_array() const
+    {
+        return false;
+    }
+
+    template <class CTE>
+    auto zexpression_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
         compute_cache();
         return m_cache;
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::get_array() const -> const xarray<value_type>&
+    auto zexpression_wrapper<CTE>::get_array() const -> const xarray<value_type>&
     {
         compute_cache();
         return m_cache;
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::clone() const -> self_type*
+    void zexpression_wrapper<CTE>::assign(xarray<value_type>&& rhs)
+    {
+        assign_impl(std::move(rhs));
+    }
+    
+    template <class CTE>
+    auto zexpression_wrapper<CTE>::clone() const -> self_type*
     {
         return new self_type(*this);
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    zarray_impl* zexpression_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
     {
-        compute_cache();
-        return m_cache.get_strides();
+        return strided_view_impl(slices, detail::is_xstrided_view<CTE>());
     }
 
     template <class CTE>
-    inline std::size_t zexpression_wrapper<CTE>::get_offset() const
+    inline zarray_impl* zexpression_wrapper<CTE>::strided_view_impl(xstrided_slice_vector& slices, std::true_type)
     {
-        compute_cache();
-        return m_cache.get_offset();
+        auto e = xt::strided_view(get_array(), slices);
+        return detail::build_zarray(std::move(e));
     }
 
     template <class CTE>
-    inline layout_type zexpression_wrapper<CTE>::layout() const
+    inline zarray_impl* zexpression_wrapper<CTE>::strided_view_impl(xstrided_slice_vector& slices, std::false_type)
     {
-        compute_cache();
-        return m_cache.layout();
+        auto e = xt::strided_view(m_expression, slices);
+        return detail::build_zarray(std::move(e));
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    auto zexpression_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
     {
         return m_metadata;
     }
 
     template <class CTE>
-    inline void zexpression_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    void zexpression_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
     {
         m_metadata = metadata;
     }
 
     template <class CTE>
-    inline std::size_t zexpression_wrapper<CTE>::dimension() const
+    std::size_t zexpression_wrapper<CTE>::dimension() const
     {
         return m_expression.dimension();
     }
 
     template <class CTE>
-    inline auto zexpression_wrapper<CTE>::shape() const -> const shape_type&
+    auto zexpression_wrapper<CTE>::shape() const -> const shape_type&
     {
         compute_cache();
         return m_cache.shape();
     }
 
     template <class CTE>
-    inline void zexpression_wrapper<CTE>::resize(const shape_type& shape)
+    void zexpression_wrapper<CTE>::resize(const shape_type&)
     {
-        throw std::runtime_error("cannot resize expression wrapper");
+        resize_impl();
     }
 
     template <class CTE>
-    inline void zexpression_wrapper<CTE>::resize(shape_type&& shape)
+    void zexpression_wrapper<CTE>::resize(shape_type&&)
     {
-        throw std::runtime_error("cannot resize expression wrapper");
+        resize_impl();
     }
 
     template <class CTE>
-    inline bool zexpression_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    bool zexpression_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
         return m_expression.broadcast_shape(shape, reuse_cache);
     }
@@ -503,7 +645,37 @@ namespace xt
             m_cache_initialized = true;
         }
     }
+    
+    template <class CTE>
+    template <class CT>
+    inline auto zexpression_wrapper<CTE>::assign_impl(xarray<value_type>&& rhs) -> enable_assignable_t<CT>
+    {
+        // aliasing is handled before this method, therefore we are sure that
+        // m_expression is not involved in rhs.
+        xt::noalias(m_expression) = rhs;
+    }
+    
+    template <class CTE>
+    template <class CT>   
+    inline auto zexpression_wrapper<CTE>::assign_impl(xarray<value_type>&&) -> enable_not_assignable_t<CT>
+    {
+        throw std::runtime_error("unevaluated expression is not assignable");
+    }
+    
+    template <class CTE>
+    template <class CT>
+    inline auto zexpression_wrapper<CTE>::resize_impl() -> enable_assignable_t<CT>
+    {
+        // Only wrappers on views are assignable. Resizing is a no op.
+    }
 
+    template <class CTE>
+    template <class CT>
+    inline auto zexpression_wrapper<CTE>::resize_impl() -> enable_not_assignable_t<CT>
+    {
+        throw std::runtime_error("cannot resize not assignable expression wrapper");
+    }
+    
     /*******************
      * zscalar_wrapper *
      *******************/
@@ -519,83 +691,84 @@ namespace xt
     }
 
     template <class CTE>
-    inline auto zscalar_wrapper<CTE>::get_array() -> xarray<value_type>&
+    bool zscalar_wrapper<CTE>::is_array() const
+    {
+        return false;
+    }
+
+    template <class CTE>
+    auto zscalar_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
         return m_array;
     }
 
     template <class CTE>
-    inline auto zscalar_wrapper<CTE>::get_array() const -> const xarray<value_type>&
+    auto zscalar_wrapper<CTE>::get_array() const -> const xarray<value_type>&
     {
         return m_array;
     }
 
     template <class CTE>
-    inline auto zscalar_wrapper<CTE>::clone() const -> self_type*
+    void zscalar_wrapper<CTE>::assign(xarray<value_type>&&)
+    {
+        throw std::runtime_error("scalar cannot be assigned an array");
+    }
+
+    template <class CTE>
+    auto zscalar_wrapper<CTE>::clone() const -> self_type*
     {
         return new self_type(*this);
     }
 
     template <class CTE>
-    inline auto zscalar_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    zarray_impl* zscalar_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
     {
-        return m_array.strides();
+        auto e = xt::strided_view(m_array, slices);
+        return detail::build_zarray(std::move(e));
     }
 
     template <class CTE>
-    inline std::size_t zscalar_wrapper<CTE>::get_offset() const
-    {
-        return 0;
-    }
-
-    template <class CTE>
-    inline layout_type zscalar_wrapper<CTE>::layout() const
-    {
-        return m_array.layout();
-    }
-
-    template <class CTE>
-    inline auto zscalar_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    auto zscalar_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
     {
         return m_metadata;
     }
 
     template <class CTE>
-    inline void zscalar_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    void zscalar_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
     {
         m_metadata = metadata;
     }
 
     template <class CTE>
-    inline std::size_t zscalar_wrapper<CTE>::dimension() const
+    std::size_t zscalar_wrapper<CTE>::dimension() const
     {
         return m_array.dimension();
     }
 
     template <class CTE>
-    inline auto zscalar_wrapper<CTE>::shape() const -> const shape_type&
+    auto zscalar_wrapper<CTE>::shape() const -> const shape_type&
     {
         return m_array.shape();
     }
 
     template <class CTE>
-    inline void zscalar_wrapper<CTE>::resize(const shape_type&)
+    void zscalar_wrapper<CTE>::resize(const shape_type&)
     {
         throw std::runtime_error("Cannot resize scalar wrapper");
     }
 
     template <class CTE>
-    inline void zscalar_wrapper<CTE>::resize(shape_type&&)
+    void zscalar_wrapper<CTE>::resize(shape_type&&)
     {
         throw std::runtime_error("Cannot resize scalar wrapper");
     }
 
     template <class CTE>
-    inline bool zscalar_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    bool zscalar_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
         return m_array.broadcast_shape(shape, reuse_cache);
     }
-
+    
     /******************
      * zarray_wrapper *
      ******************/
@@ -639,83 +812,98 @@ namespace xt
     }
 
     template <class CTE>
-    inline auto zarray_wrapper<CTE>::get_array() -> xarray<value_type>&
+    bool zarray_wrapper<CTE>::is_array() const
+    {
+        return true;
+    }
+
+    template <class CTE>
+    auto zarray_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
         return detail::zarray_wrapper_helper<value_type>::get_array(m_array);
     }
 
     template <class CTE>
-    inline auto zarray_wrapper<CTE>::get_array() const -> const xarray<value_type>&
+    auto zarray_wrapper<CTE>::get_array() const -> const xarray<value_type>&
     {
         return m_array;
     }
 
     template <class CTE>
-    inline auto zarray_wrapper<CTE>::clone() const -> self_type*
+    void zarray_wrapper<CTE>::assign(xarray<value_type>&& rhs)
+    {
+        assign_impl(std::move(rhs));
+    }
+
+    template <class CTE>
+    auto zarray_wrapper<CTE>::clone() const -> self_type*
     {
         return new self_type(*this);
     }
 
     template <class CTE>
-    inline auto zarray_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    zarray_impl* zarray_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
     {
-        return m_array.strides();
+        auto e = xt::strided_view(m_array, slices);
+        return detail::build_zarray(std::move(e));
     }
 
     template <class CTE>
-    inline std::size_t zarray_wrapper<CTE>::get_offset() const
-    {
-        return 0;
-    }
-
-    template <class CTE>
-    inline layout_type zarray_wrapper<CTE>::layout() const
-    {
-        return m_array.layout();
-    }
-
-    template <class CTE>
-    inline auto zarray_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    auto zarray_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
     {
         return m_metadata;
     }
 
     template <class CTE>
-    inline void zarray_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    void zarray_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
     {
         m_metadata = metadata;
     }
 
     template <class CTE>
-    inline std::size_t zarray_wrapper<CTE>::dimension() const
+    std::size_t zarray_wrapper<CTE>::dimension() const
     {
         return m_array.dimension();
     }
 
     template <class CTE>
-    inline auto zarray_wrapper<CTE>::shape() const -> const shape_type&
+    auto zarray_wrapper<CTE>::shape() const -> const shape_type&
     {
         return m_array.shape();
     }
 
     template <class CTE>
-    inline void zarray_wrapper<CTE>::resize(const shape_type& shape)
+    void zarray_wrapper<CTE>::resize(const shape_type& shape)
     {
         detail::zarray_wrapper_helper<value_type>::resize(m_array, shape);
     }
 
     template <class CTE>
-    inline void zarray_wrapper<CTE>::resize(shape_type&& shape)
+    void zarray_wrapper<CTE>::resize(shape_type&& shape)
     {
         detail::zarray_wrapper_helper<value_type>::resize(m_array, std::move(shape));
     }
 
     template <class CTE>
-    inline bool zarray_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    bool zarray_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
         return m_array.broadcast_shape(shape, reuse_cache);
     }
+    
+    template <class CTE>
+    template <class CT>
+    inline detail::enable_const_t<CT> zarray_wrapper<CTE>::assign_impl(xarray<value_type>&&)
+    {
+        throw std::runtime_error("const array is not assignable");
+    }
 
+    template <class CTE>
+    template <class CT>
+    inline detail::disable_const_t<CT> zarray_wrapper<CTE>::assign_impl(xarray<value_type>&& rhs)
+    {
+        m_array = std::move(rhs);
+    }
+   
     /********************
      * zchunked_wrapper *
      ********************/
@@ -737,94 +925,94 @@ namespace xt
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::get_array() -> xarray<value_type>&
+    bool zchunked_wrapper<CTE>::is_array() const
+    {
+        return false;
+    }
+
+    template <class CTE>
+    auto zchunked_wrapper<CTE>::get_array() -> xarray<value_type>&
     {
         compute_cache();
         return m_cache;
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::get_array() const -> const xarray<value_type>&
+    auto zchunked_wrapper<CTE>::get_array() const -> const xarray<value_type>&
     {
         compute_cache();
         return m_cache;
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::clone() const -> self_type*
+    void zchunked_wrapper<CTE>::assign(xarray<value_type>&& rhs)
+    {
+        assign_impl(std::move(rhs));
+    }
+
+    template <class CTE>
+    auto zchunked_wrapper<CTE>::clone() const -> self_type*
     {
         return new self_type(*this);
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::get_strides() const -> const dynamic_shape<std::ptrdiff_t>&
+    zarray_impl* zchunked_wrapper<CTE>::strided_view(xstrided_slice_vector& slices)
     {
-        if (!m_strides_initialized)
-        {
-            m_strides = detail::get_strides<XTENSOR_DEFAULT_TRAVERSAL>(m_chunked_array);
-            m_strides_initialized = true;
-        }
-        return m_strides;
+        auto e = xt::strided_view(m_chunked_array, slices);
+        return detail::build_zarray(std::move(e));
     }
 
     template <class CTE>
-    inline std::size_t zchunked_wrapper<CTE>::get_offset() const
-    {
-        return 0;
-    }
-
-    template <class CTE>
-    inline layout_type zchunked_wrapper<CTE>::layout() const
-    {
-        return m_chunked_array.layout();
-    }
-
-    template <class CTE>
-    inline auto zchunked_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
+    auto zchunked_wrapper<CTE>::get_metadata() const -> const nlohmann::json&
     {
         return m_metadata;
     }
 
     template <class CTE>
-    inline void zchunked_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
+    void zchunked_wrapper<CTE>::set_metadata(const nlohmann::json& metadata)
     {
         m_metadata = metadata;
     }
 
     template <class CTE>
-    inline std::size_t zchunked_wrapper<CTE>::dimension() const
+    std::size_t zchunked_wrapper<CTE>::dimension() const
     {
         return m_chunked_array.dimension();
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::shape() const -> const shape_type&
+    auto zchunked_wrapper<CTE>::shape() const -> const shape_type&
     {
         return m_chunked_array.shape();
     }
 
     template <class CTE>
-    inline void zchunked_wrapper<CTE>::resize(const shape_type&)
+    void zchunked_wrapper<CTE>::resize(const shape_type&)
     {
-        //TODO: check this semantic
-        throw std::runtime_error("Cannot resize chunked wrapper");
+        // This function is called because zarray (and therefore
+        // all the wrappers) implements the container semantic,
+        // whatever it wraps (and it cannot be different, since
+        // the type of the wrappee is erased).
+        // We can see this as a mapping container_semantic => chunked_semantic,
+        // meaning this function call must be authorized, and must not do
+        // anything.
     }
 
     template <class CTE>
-    inline void zchunked_wrapper<CTE>::resize(shape_type&&)
+    void zchunked_wrapper<CTE>::resize(shape_type&&)
     {
-        //TODO: check this semantic
-        throw std::runtime_error("Cannot resize chunked wrapper");
+        // See comment above.
     }
 
     template <class CTE>
-    inline bool zchunked_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
+    bool zchunked_wrapper<CTE>::broadcast_shape(shape_type& shape, bool reuse_cache) const
     {
         return m_chunked_array.broadcast_shape(shape, reuse_cache);
     }
 
     template <class CTE>
-    inline auto zchunked_wrapper<CTE>::chunk_shape() const -> const shape_type&
+    auto zchunked_wrapper<CTE>::chunk_shape() const -> const shape_type&
     {
         return m_chunk_shape;
     }
@@ -839,56 +1027,18 @@ namespace xt
         }
     }
 
-    /******************
-     * zarray builder *
-     ******************/
-
-    namespace detail
+    template <class CTE>
+    template <class CT>
+    inline detail::enable_const_t<CT> zchunked_wrapper<CTE>::assign_impl(xarray<value_type>&&)
     {
-        template <class E>
-        struct is_xarray : std::false_type
-        {
-        };
+        throw std::runtime_error("const array is not assignable");
+    }
 
-        template <class T, layout_type L, class A, class SA>
-        struct is_xarray<xarray<T, L, A, SA>> : std::true_type
-        {
-        };
-
-        template <class E>
-        struct is_chunked_array : std::false_type
-        {
-        };
-
-        template <class CS>
-        struct is_chunked_array<xchunked_array<CS>> : std::true_type
-        {
-        };
-
-        template <class E>
-        struct zwrapper_builder
-        {
-            using closure_type = xtl::closure_type_t<E>;
-            using wrapper_type = std::conditional_t<is_xarray<std::decay_t<E>>::value,
-                                                    zarray_wrapper<closure_type>,
-                                                    std::conditional_t<is_chunked_array<std::decay_t<E>>::value,
-                                                                       zchunked_wrapper<closure_type>,
-                                                                       zexpression_wrapper<closure_type>
-                                                                      >
-                                                    >;
-
-            template <class OE>
-            static wrapper_type* run(OE&& e)
-            {
-                return new wrapper_type(std::forward<OE>(e));
-            }
-        };
-
-        template <class E>
-        inline auto build_zarray(E&& e)
-        {
-            return zwrapper_builder<E>::run(std::forward<E>(e));
-        }
+    template <class CTE>
+    template <class CT>
+    inline detail::disable_const_t<CT> zchunked_wrapper<CTE>::assign_impl(xarray<value_type>&& rhs)
+    {
+        xt::noalias(m_chunked_array) = rhs;
     }
 }
 
