@@ -36,7 +36,17 @@ namespace xt
             static void assign_data(E1& e1, const E2& e2)
             {
                 using value_type = typename E2::value_type;
-                e1.template get_array<value_type>() = e2;
+                using array_type = ztyped_array<value_type>;
+                array_type& ar = dynamic_cast<array_type&>(e1.get_implementation());
+                if (ar.is_array())
+                {
+                    xt::noalias(ar.get_array()) = e2;
+                }
+                else
+                {
+                    xarray<value_type> tmp(e2);
+                    ar.assign(std::move(tmp));
+                }
             }
         };
     }
@@ -107,6 +117,20 @@ namespace xt
         bool linear_assign = detail::xis_linear_assign<E1, E2>::run(e1.derived_cast(), e2.derived_cast());
         assign_data(e1, e2, linear_assign);
     }
+
+    template <class T, class E2>
+    inline void zassign_wrapped_expression(ztyped_array<T>& lhs, const xexpression<E2>& rhs)
+    {
+        if (lhs.is_array())
+        {
+            zassign_wrapped_expression(lhs.get_array(), rhs);
+        }
+        else
+        {
+            xarray<T> tmp(rhs);
+            lhs.assign(std::move(tmp));
+        }
+    }
 }
 
 #endif
